# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
from cvprac.cvp_client_errors import CvpApiError
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()
from cvprac.cvp_client import CvpClient

with open("cvaas.tok") as f:
    token = f.read().strip('\n')

clnt = CvpClient()
clnt.connect(nodes=['www.arista.io'], username='', password='', is_cvaas=True, api_token=token)

user_info = clnt.api.get_user('kishore')
print (user_info)



from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username="username",password="password")

ts = "2021-11-19T15:04:05.0Z" # rfc3339 time
uri = "/api/v3/services/compliancecheck.Compliance/GetConfig"

# Fetch the inventory
inventory = clnt.api.get_inventory()

# Iterate through all devices and get the running-config at the specified time for each device
for device in inventory:
    sn = device['serialNumber']
    data = {"request":{
        "device_id": sn,
        "timestamp": ts,
        "type":"RUNNING_CONFIG"
        }
    }
    try:
        resultRunningConfig = clnt.post(uri, data=data)
        for idx in resultRunningConfig:
            if 'config' in idx:
                result = idx['config']
                break
        with open(device['hostname']+'.cfg','w') as f:
            f.write(result)
    except Exception as e:
        print("Not able to get configuration for device {} - exception {}".format(device['fqdn'], e))



# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
with open("token.tok") as f:
    token = f.read().strip('\n')

clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username='',password='',api_token=token)

container = clnt.api.get_container_by_name('TP_LEAFS') # container object

app_name = "my app" # can be any string

device = {"key":"00:1c:73:c5:4c:87", "fqdn":"co633.ire.aristanetworks.com"}

move_device_to_container(app_name, device, container)



# Copyright (c) 2023 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
import argparse
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username="username", password="password")

parser = argparse.ArgumentParser(
    description='Get the list of devices and containers a configlet is attached to')
parser.add_argument('-c', '--configlet', required=True, help='The name of the configlet')
args = parser.parse_args()

configlet_name = args.configlet
devices = clnt.api.get_applied_devices(configlet_name)

containers = clnt.api.get_applied_containers(configlet_name)
print(f"Total number of devices {configlet_name} is attached to: {devices['total']}\n")
print(f"Total number of containers {configlet_name} is attached to: {containers['total']}\n")
col1 = "Device FQDN/hostname"
col2 = "IP Address"
print(f"{col1:<40}{col2:<40}")
print("="*80)
for device in devices['data']:
    print(f"{device['hostName']:<40}{device['ipAddress']}")

print("\nList of containers:\n")
for container in containers['data']:
    print(container['containerName'])



# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3

requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username="username",password="password")

configletName = 'cvprac_example2'

device_name = "tp-avd-leaf1"
device = clnt.api.get_device_by_name(device_name)

configlet = clnt.api.get_configlet_by_name(configletName)

clnt.api.apply_configlets_to_device("", device, [configlet])



# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
from cvprac.cvp_client_errors import CvpApiError
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision using user/password (on-prem only)
clnt = CvpClient()
clnt.connect(['cvp1'],'username', 'password')

username = "cvprac2"
clnt.api.svc_account_delete(username)



# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

### Compliance Code description
compliance = {"0000":"Configuration is in sync",
              "0001": "Config is out of sync",
              "0002": "Image is out of sync",
              "0003": "Config & image out of sync",
              "0004": "Config, Image and Device time are in sync",
              "0005": "Device is not reachable",
              "0006": "The current EOS version on this device is not supported by CVP. Upgrade the device to manage.",
              "0007": "Extensions are out of sync",
              "0008": "Config, Image and Extensions are out of sync",
              "0009": "Config and Extensions are out of sync",
              "0010": "Image and Extensions are out of sync",
              "0011": "Unauthorized User",
              "0012": "Config, Image, Extension and Device time are out of sync",
              "0013": "Config, Image and Device time are out of sync",
              "0014": "Config, Extensions and Device time are out of sync",
              "0015": "Image, Extensions and Device time are out of sync",
              "0016": "Config and Device time are out of sync",
              "0017": "Image and Device time are out of sync",
              "0018": "Extensions and Device time are out of sync",
              "0019": "Device time is out of sync"
}

# Create connection to CloudVision using Service account token
with open("token.tok") as f:
    token = f.read().strip('\n')

clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username='',password='',api_token=token)

def check_devices_under_container(client, container):
    ''' container is the container ID '''

    nodeId = container['key']
    nodeName = container['name']
    api = '/ztp/getAllNetElementList.do?'
    queryParams = "nodeId={}&queryParam=&nodeName={}&startIndex=0&endIndex=0&contextQueryParam=&ignoreAdd=false&useCache=true".format(nodeId, nodeName)
    return client.get(api + queryParams)


container = clnt.api.get_container_by_name('TP_LEAFS')

devices = (check_devices_under_container(clnt,container))

for device in devices['netElementList']:
    code = device['complianceCode']
    print(device['fqdn'], ' ', code,' ', compliance[code])



# Copyright (c) 2022 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

# This script can be run as a cronjob to periodically reconcile all devices
# that are out of configuration compliance in environments where the running-config
# is still modified via the CLI often.
from cvprac.cvp_client import CvpClient
import ssl
from datetime import datetime
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()
clnt = CvpClient()
clnt.set_log_level(log_level='WARNING')

# Reading the service account token from a file
with open("token.tok") as f:
    token = f.read().strip('\n')

clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username='',password='',api_token=token)

inventory = clnt.api.get_inventory()

compliance = {"0001": "Config is out of sync",
        "0003": "Config & image out of sync",
        "0004": "Config, Image and Device time are in sync",
        "0005": "Device is not reachable",
        "0008": "Config, Image and Extensions are out of sync",
        "0009": "Config and Extensions are out of sync",
        "0012": "Config, Image, Extension and Device time are out of sync",
        "0013": "Config, Image and Device time are out of sync",
        "0014": "Config, Extensions and Device time are out of sync",
        "0016": "Config and Device time are out of sync"
    }

non_compliants = []
taskIds = []
for device in inventory:
    if device['complianceCode'] in compliance.keys():
        # create a list of non-compliant devices for reporting purposes
        non_compliants.append(device['hostname'])
        dev_mac = device['systemMacAddress']
        # check if device already has reconciled config and save the key if it does
        try:
            configlets = clnt.api.get_configlets_by_device_id(dev_mac)
            for configlet in configlets:
                if configlet['reconciled'] == True:
                    configlet_key = configlet['key']
                    break
                else:
                    configlet_key = ""
            rc = clnt.api.get_device_configuration(dev_mac)
            name = 'RECONCILE_' + device['serialNumber']
            update = clnt.api.update_reconcile_configlet(dev_mac, rc, configlet_key, name, True)
            # if the device had no reconciled config, it means we need to append the reconciled
            # configlet to the list of applied configlets on the device
            if configlet_key == "":
                addcfg = clnt.api.apply_configlets_to_device("auto-reconciling",device,[update['data']])
                clnt.api.cancel_task(addcfg['data']['taskIds'][0])            
        except Exception as e:
            continue
print(f"The non compliant devices were: {str(non_compliants)}")



# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

# This script is an example on provisioning registered devices in CloudVision that is based on
# Arista Test Drive (ATD) and similar to what the ansible playbooks do in
# https://github.com/arista-netdevops-community/atd-avd.
# It does the following:
# - creates and uploads configlets,
# - creates the container hierarchy in Network Provisiong
# - moves the devices to their target containers
# - assigns the configlets to the devices
# - creates a change control from the genereated tasks
# - approves and executes the change control

import uuid
import time
import ssl
from datetime import datetime
from cvprac.cvp_client import CvpClient
ssl._create_default_https_context = ssl._create_unverified_context

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(['cvp1'],'username', 'password')

# Create container topology
container_name = "DC1_LEAFS"
container_topology = [{"containerName": "ATD_FABRIC", "parentContainerName": 'Tenant'},
                      {"containerName": "ATD_LEAFS", "parentContainerName": 'ATD_FABRIC'},
                      {"containerName": "pod1", "parentContainerName": 'ATD_LEAFS'},
                      {"containerName": "pod2", "parentContainerName": 'ATD_LEAFS'},
                      {"containerName": "ATD_SERVERS", "parentContainerName": 'ATD_FABRIC'},
                      {"containerName": "ATD_SPINES", "parentContainerName": 'ATD_FABRIC'},
                      {"containerName": "ATD_TENANT_NETWORKS", "parentContainerName": 'ATD_FABRIC'}]
for container in container_topology:
    try:
        container_name = container['containerName']
        # Get parent container information
        parent = clnt.api.get_container_by_name(container['parentContainerName'])
        print(f'Creating container {container_name}\n')
        clnt.api.add_container(container_name,parent["name"],parent["key"])
    except Exception as e:
        if "Data already exists in Database" in str(e):
            print ("Container already exists, continuing...")

# Create device mappers
devices = [{'deviceName': "leaf1",
            'configlets': ["BaseIPv4_Leaf1", "AVD_leaf1"],
            "parentContainerName": "pod1"},
           {'deviceName': "leaf2",
            'configlets': ["BaseIPv4_Leaf2", "AVD_leaf2"],
            "parentContainerName": "pod1"},
           {'deviceName': "leaf3",
            'configlets': ["BaseIPv4_Leaf3", "AVD_leaf3"],
            "parentContainerName": "pod2"},
           {'deviceName': "leaf4",
            'configlets': ["BaseIPv4_Leaf4", "AVD_leaf4"],
            "parentContainerName": "pod2"},
           {'deviceName': "spine1",
            'configlets': ["BaseIPv4_Spine1", "AVD_spine1"],
            "parentContainerName": "ATD_SPINES"},
           {'deviceName': "spine2",
            'configlets': ["BaseIPv4_Spine2", "AVD_spine2"],
            "parentContainerName": "ATD_SPINES"}]

task_list = []
for device in devices:
    # Load the AVD configlets from file
    with open("./configlets/AVD_" + device['deviceName'] + ".cfg", "r") as file:
        configlet_file = file.read()
    avd_configlet_name = device['configlets'][1]
    base_configlet_name = device['configlets'][0] # preloaded configlet in an ATD environment
    container_name = device['parentContainerName']
    base_configlet = clnt.api.get_configlet_by_name(base_configlet_name)
    configlets = [base_configlet]
    # Update the AVD configlets if they exist, otherwise upload them from the configlets folder
    print (f"Creating configlet {avd_configlet_name} for {device['deviceName']}\n")
    try:
        configlet = clnt.api.get_configlet_by_name(avd_configlet_name)
        clnt.api.update_configlet(configlet_file, configlet['key'], avd_configlet_name)
        configlets.append(configlet)
    except:
        clnt.api.add_configlet(avd_configlet_name, configlet_file)
        configlet = clnt.api.get_configlet_by_name(avd_configlet_name)
        configlets.append(configlet)
    # Get device data
    device_data = clnt.api.get_device_by_name(device['deviceName'] + ".atd.lab")
    # Get the parent container data for the device
    container = clnt.api.get_container_by_name(container_name)
    device_name = device['deviceName']
    print(f"Moving device {device_name} to container {container_name}\n")
    # The move action will create the task first, however if the devices are already in the target
    # container, for instance if the script was run multiple times than the move action will
    # not generate a task anymore, therefore it's better to create the task list from the
    # Update Config action which will reuse the Move Device action's task if one exists,
    # otherwise will create a new one.
    move = clnt.api.move_device_to_container("python", device_data, container)
    apply_configlets = clnt.api.apply_configlets_to_device("", device_data, configlets)
    task_list = task_list + apply_configlets['data']['taskIds']

print(f"Generated task IDs are: {task_list}\n")

# Generate unique ID for the change control
cc_id = str(uuid.uuid4())
cc_name = f"Change_{datetime.now().strftime('%Y%m%d_%H%M%S')}"

print("Creating Change control with the list of tasks")
clnt.api.change_control_create_for_tasks(cc_id, cc_name, task_list, series=False)

print("Approving Change Control")
# adding a few seconds sleep to avoid small time diff between the local system and CVP
time.sleep(2)
approve_note = "Approving CC via cvprac"
clnt.api.change_control_approve(cc_id, notes=approve_note)

# Start the change control
print("Executing Change Control...")
start_note = "Start the CC via cvprac"
clnt.api.change_control_start(cc_id, notes=start_note)



# Copyright (c) 2022 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username="username",password="password")

def main():

   print('Retrieving configlets ...')

   inventory = clnt.api.get_inventory()
   data = clnt.api.get_configlets_and_mappers()['data']
   print(data)

   print('Number of configlets:', len(data['configlets']))

   searchAgain = True
   while searchAgain:
        try:
            search = input( "\nEnter Config Line: " )
            print(f"\n\n\'{search}\' has been found in following configlets:\n\n")
            print(f"{'Hostname':<30}{'Serial number':<50}{'MAC address':<30}{'Configlets':<40}")
            print("=" * 150)
            for i in inventory:
                device = i['hostname']
                device_sn = i['serialNumber']
                device_mac = i['systemMacAddress']
                configlet_list = []
                for c in data['configlets']:
                    for g in data['generatedConfigletMappers']:
                        if device_mac == g['netElementId'] and c['key'] == g['configletBuilderId'] and search in c['config']:
                            configlet_list.append(c['name'])
                    for k in data['configletMappers']:
                        if device_mac == k['objectId'] and c['key'] == k['configletId'] and search in c['config']:
                            configlet_list.append(c['name'])
                configlet_list_final = ",".join(configlet_list)
                if len(configlet_list) > 0:                
                    print(f"{device:<30}{device_sn:<50}{device_mac:<30}{configlet_list_final:<30}")

        except KeyboardInterrupt:
            print('\nExiting... \n')
            return

if __name__ == '__main__':
   main()




# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.
#
# NOTE: The following example is using the new Change Control Resource APIs supported in 2021.2.0 or newer and in CVaaS.
# For CVaaS service-account token based auth has to be used.

from cvprac.cvp_client import CvpClient
import ssl
import uuid
from datetime import datetime
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(['cvp1'],'username', 'password')

# Generate change control id and change control name
cc_id = str(uuid.uuid4())
name = f"Change_{datetime.now().strftime('%Y%m%d_%H%M%S')}"

# Select the tasks and create a CC where all tasks will be run in parallel
tasks = ["1249","1250","1251","1252"]
clnt.api.change_control_create_for_tasks(cc_id, name, tasks, series=False)

# Approve the change control
approve_note = "Approving CC via cvprac"
clnt.api.change_control_approve(cc_id, notes=approve_note)

# # Schedule the change control
# # Executing scheduled CCs might only work post 2021.3.0+
# schedule_note = "Scheduling CC via cvprac"
# schedule_time = "2021-12-23T03:17:00Z"
# clnt.api.change_control_schedule(cc_id,schedule_time,notes=schedule_note)

# Start the change control
start_note = "Start the CC via cvprac"
clnt.api.change_control_start(cc_id, notes=start_note)


#
# Copyright (c) 2016, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
""" This module provides a RESTful API client for Cloudvision(R) Portal (CVP)
    which can be used for building applications that work with Arista CVP.
"""
import io
from os import path, walk

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from cvprac import __version__, __author__


def find_modules(pkg):
    ''' Return all modules from the pkg
    '''
    modules = [pkg]
    for dirname, dirnames, _ in walk(pkg):
        for subdirname in dirnames:
            modules.append(path.join(dirname, subdirname))
    return modules


def get_long_description():
    ''' Get the long description from README.rst if it exists.
        Null string is returned if README.rst is non-existent
    '''
    long_description = ''
    here = path.abspath(path.dirname(__file__))
    try:
        with io.open(path.join(here, 'README.md'), encoding='utf-8') as file_hdl:
            long_description = file_hdl.read()
    except IOError:
        pass
    return long_description


setup(
    name='cvprac',
    version=__version__,
    description='Arista Cloudvision(R) Portal Rest API Client written in python',
    long_description=get_long_description(),
    long_description_content_type='text/markdown',
    author=__author__,
    author_email='eosplus-dev@arista.com',
    url='https://github.com/aristanetworks/cvprac',
    download_url='https://github.com/aristanetworks/cvprac/tarball/%s' % __version__,
    license='BSD-3',
    packages=find_modules('cvprac'),

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',

        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Networking',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: BSD License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],

    # What does your project relate to?
    keywords='networking CloudVision development rest api',

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests[socks]>=2.27.0', 'packaging>=23.2'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev]
    extras_require={
        'dev': ['check-manifest', 'pep8', 'pyflakes', 'pylint', 'coverage',
                'pyyaml'],
    },
)



# Copyright (c) 2020 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(['cvp1'],'username', 'password')

image_name = "vEOS-4.26.0.1F"
image = clnt.api.get_image_bundle_by_name(image_name)

container_name = "TP_FABRIC"
container = clnt.api.get_container_by_name(container_name)

clnt.api.remove_image_from_container(image, container)



# Copyright (c) 2020 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(['cvp1'],'username', 'password')

# Get parent container information
parent = clnt.api.get_container_by_name("ContainerA")

# Create new container ContainerB under ContainerA

clnt.api.add_container("ContainerB",parent["name"],parent["key"])
